local SR_UnitAuras = SpellReminder:NewModule("SR_UnitAuras","AceEvent-3.0", "AceBucket-3.0")

function SR_UnitAuras:OnDisable()
	self:UnregisterAllEvents()
end

function SR_UnitAuras:OnEnable()
	self:RegisterEvent("PLAYER_FOCUS_CHANGED",function() self:ProcessAuras("focus")	end)
	self:RegisterEvent("PLAYER_TARGET_CHANGED",function() self:ProcessAuras("target") end)
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT",function() self:ProcessAuras("mouseover") end)

	self:RegisterBucketEvent("UNIT_AURA",0.25,function(units)
		for unit in pairs(units) do
			self:ProcessAuras(unit)
		end
	end)

	self:RegisterEvent("UNIT_PET",function (unit)
		if unit ~= "player" then return end -- only process for players pet.
		for eventName,info in pairs(spellTracker.active) do -- first clear all CD timers, this will stop  multiple bars when zoning
			if info.tGUID == "PET" then
				self:CleanUp(eventName)--self:OnDurationEnded(eventName,true)
			end
		end
		self:ProcessAuras("pet")
	end)

	self:ProcessAuras("player")
	self:ProcessAuras("pet")
end

function SR_UnitAuras:ProcessAuras(unit)
	local spellTracker = SpellReminder.track
	local unitGUID = UnitGUID(unit)
	local now = GetTime()

	--[[ check throttle
	if unitThrottle[unitGUID] ~= nil and now - unitThrottle[unitGUID] < 3 then
		self:DebugPrint("Throttled: "..unit)
		return
	end
	unitThrottle[unitGUID] = now  --]]
	
	SpellReminder:DebugPrint("ProcessAuras "..unit)
	local targetName,targetRealm = UnitName(unit)
	local playerGUID, petGUID, vehicleGUID = UnitGUID("player"), UnitGUID("pet"), UnitGUID("vehicle")
	if targetName == nil or targetName == "Unknown" then return end
	if UnitIsDead(unit) then return end
	if targetRealm ~= nil then targetName = string.gsub(targetName,'-'..targetRealm,'') end
	--if unitGUID == vehicleGUID then
	--	unitGUID = "VEHICLE"
	--else
	if unitGUID == petGUID then
		unitGUID = "PET"
	end
	local hasAura = {}
	
	do	-- BUFFS
		local i = 1
		while UnitExists(unit) do
			local name, rank, iconTexture, applications, spellType, duration, timeEnd, timeLeft, caster, isMine
			name, rank, iconTexture, applications, spellType, duration, timeEnd, caster = UnitBuff(unit, i)
			isMine = caster == "player" or caster == "pet" --or caster == "vehicle"
			timeLeft = (timeEnd and (timeEnd - now)) or nil
			
			if (name == nil) then break end
			
			local eventName = SpellReminder:GetEventName(name,unitGUID)
			while hasAura[eventName] do local i = (i or 0)+1; eventName = SpellReminder:GetEventName(name..i,unitGUID) end -- this line allows multiple buffs of the same spell based on aura, useful for dual mongoose proc
			if duration ~= nil and duration >0 and timeLeft ~= nil and timeLeft > 0 and isMine then
				duration = math.ceil(duration)
				local timeStarted = now - (duration - timeLeft)
				local castingInfo = SpellReminder:GetCastingObject(name,nil,rank,targetName,unitGUID,timeStarted,duration,nil,eventName,nil,"buff",iconTexture)
				castingInfo["applications"] = applications
				self:TrackSpell(castingInfo,true)
				SpellReminder:SetSpellCategory(name,'Buffs')
				hasAura[eventName] = true
			end
			i = i + 1
		end
	end
	
	do	-- DEBUFFS
		local i = 1
		while UnitExists(unit) do
			local name, rank, iconTexture, applications, spellType, duration, timeEnd, timeLeft, caster, isMine
			name, rank, iconTexture, applications, spellType, duration, timeEnd, caster = UnitDebuff(unit, i)
			isMine = caster == "player" or caster == "pet" --or caster == "vehicle"
			timeLeft = (timeEnd and (timeEnd - now)) or nil
			
			if (name == nil) then break end
			
			local eventName = SpellReminder:GetEventName(name,unitGUID)
			while hasAura[eventName] do local i = (i or 0)+1; eventName = SpellReminder:GetEventName(name..i,unitGUID) end -- this line allows multiple buffs of the same spell based on aura, useful for dual mongoose proc
			if duration ~= nil and duration >0 and timeLeft ~= nil and timeLeft > 0 and isMine then
				duration = math.ceil(duration)
				local timeStarted = now - (duration - timeLeft)
				local castingInfo = SpellReminder:GetCastingObject(name,nil,rank,targetName,unitGUID,timeStarted,duration,nil,eventName,nil,"debuff",iconTexture)
				castingInfo["applications"] = applications
				self:TrackSpell(castingInfo,true)
				SpellReminder:SetSpellCategory(name,'Debuffs')
				hasAura[eventName] = true
			end
			i = i + 1
		end
	end
	
	-- remove bars if auras are no longer active
	for eventName,castingInfo in pairs(spellTracker.active) do
		if castingInfo.module == self:GetName() then
			if (castingInfo.tGUID == unitGUID) and (not hasAura[eventName]) and (castingInfo.n ~= UnitName("pet")) then
				SpellReminder:OnDurationEnded(eventName)
			end
		end
	end
	
	hasAura = nil
end
